
#include "TArtAnaLoopManager.hh"

#include <iostream>

#include "TThread.h"
#include "TDirectory.h"

#include "TArtAnaLoop.hh"
#include "TArtAnaFile.hh"

//_______________________________________________________________________________________

TThread* book(TArtAnaLoop* analoop, const char* anafilename, const char* filename,
	      int eventnumber, TDirectory* directory)
{return TArtAnaLoopManager::Book(analoop, anafilename, filename, eventnumber, directory);}
void start(){TArtAnaLoopManager::Start();}
void stop(){TArtAnaLoopManager::Stop();}
void end(){TArtAnaLoopManager::End();}
void status(){TArtAnaLoopManager::Status();}
TCutG* hcut(int id, const char* filename, const char* cutname){return TArtAnaLoopManager::HCut(id, filename, cutname);}

//_______________________________________________________________________________________

TArtAnaLoopManager::EAnalysisState TArtAnaLoopManager::kAnalysisState = TArtAnaLoopManager::INIT;
TThread* TArtAnaLoopManager::fThread = 0;

TThread* TArtAnaLoopManager::Book(TArtAnaLoop* analoop,
				  const char* anafilename, const char* filename,
				  int eventnumber, TDirectory* directory)
{
  if(fThread != 0){
    End();
    delete fThread;
    fThread = 0;
  }

  if(anafilename != "" && anafilename != 0 && !analoop->OpenAnaFile(anafilename)) return 0;
  TArtAnaLoopStatic::SetAnaLoop(analoop);
  TArtAnaLoopStatic::SetFileName(filename);
  TArtAnaLoopStatic::SetEventNumber(eventnumber);
  TArtAnaLoopStatic::SetDirectory(directory);

  void *(*fpt)(void*) = TArtAnaLoopStatic::AnaLoopStatic;

  return fThread = new TThread((void *(*)(void*))fpt, (void*)0);
}

void TArtAnaLoopManager::Start()
{
  if(!fThread){
    std::cout << "maybe missing to book" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n start analoop\n" << std::endl;    
    fThread->Run();
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n ...running\n" << std::endl;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n resume analoop\n" << std::endl;    
    kAnalysisState = RUNNING;
    fThread->Run();
  }else if(kAnalysisState == END){
    std::cout << "\n start analoop\n" << std::endl;
    kAnalysisState = INIT;
    fThread->Run();
  }else if(kAnalysisState == ERROR){
    std::cout << "\n start analoop\n" << std::endl;    
    fThread->Run();
  }
}

void TArtAnaLoopManager::Stop()
{
  if(!fThread){
    std::cout << "maybe missing to book" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n stop analoop\n" << std::endl;
    kAnalysisState = SUSPENDED;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n ...suspended\n" << std::endl;    
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }else if(kAnalysisState == ERROR){
    std::cout << "\n ...error\n" << std::endl;    
    fThread->Run();
  }
}

void TArtAnaLoopManager::End()
{
  if(!fThread){
    std::cout << "maybe miss to book" << std::endl;
    return;
  }

  if(kAnalysisState == INIT){
    std::cout << "\n ...init\n" << std::endl;
  }else if(kAnalysisState == RUNNING){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
  }else if(kAnalysisState == SUSPENDED){
    std::cout << "\n end analoop\n" << std::endl;
    kAnalysisState = END;
    fThread->Run();
  }else if(kAnalysisState == END){
    std::cout << "\n ...end\n" << std::endl;
  }else if(kAnalysisState == ERROR){
    std::cout << "\n ...error\n" << std::endl;    
    fThread->Run();
  }
}

void TArtAnaLoopManager::Status()
{
  std::cout << std::endl;

  TString state;
  switch (kAnalysisState){
  case INIT:
    state = "INIT"; break;
  case RUNNING:
    state = "RUNNING"; break;
  case SUSPENDED:
    state = "SUSPENDED"; break;
  case END:
    state = "END"; break;
  case ERROR:
    state = "ERROR"; break;
  }
  std::cout << " Analysis State: " << state << std::endl;
  std::cout << " File Name: " << TArtAnaLoopStatic::GetFileName() << std::endl;
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  TString classname("");
  int eventnumber(0);
  if(analoop != 0){
    classname = analoop->ClassName();
    eventnumber = analoop->GetEventNumber();
  }
  std::cout << " Class Name: " << classname << std::endl;
  std::cout << " Event Number: " << eventnumber << std::endl;

  std::cout << std::endl;
}

TCutG* TArtAnaLoopManager::HCut(int id, const char* filename, const char* cutname)
{
  TArtAnaLoop* analoop = TArtAnaLoopStatic::GetAnaLoop();
  if(analoop == 0) return 0;
  if(analoop->fAnaFile == 0) return 0;

  return analoop->fAnaFile->CreateCutG(id, filename, cutname);
}

TArtAnaLoopManager::~TArtAnaLoopManager()
{
  delete fThread;
  fThread = 0;
}

//_______________________________________________________________________________________

TArtAnaLoop* TArtAnaLoopStatic::fAnaLoop = 0;
TString TArtAnaLoopStatic::fFileName = "";
TDirectory* TArtAnaLoopStatic::fDirectory = 0;
int TArtAnaLoopStatic::fEventNumber = -1;

void* TArtAnaLoopStatic::AnaLoopStatic(void *)
{
  fDirectory->cd();
  if(fFileName.Length() == 0){
    fAnaLoop->AnaLoop((char*)0, fEventNumber);
  }else{
    fAnaLoop->AnaLoop(fFileName.Data(), fEventNumber);
  }    

  return 0;
}

void TArtAnaLoopStatic::SetAnaLoop(TArtAnaLoop* analoop)
{
  if(fAnaLoop) delete fAnaLoop;
  fAnaLoop = analoop;
}
void TArtAnaLoopStatic::SetFileName(const char* filename)
{
  if(filename == 0){
    fFileName = "";
  }else{
    fFileName = filename;
  }
}
void TArtAnaLoopStatic::SetEventNumber(int eventnumber){fEventNumber = eventnumber;}
void TArtAnaLoopStatic::SetDirectory(TDirectory* directory)
{
  if(directory == 0){
    fDirectory = gDirectory;
  }else{
    fDirectory = directory;
  }
}  
TArtAnaLoop* TArtAnaLoopStatic::GetAnaLoop(){return fAnaLoop;}
const char* TArtAnaLoopStatic::GetFileName(){return fFileName.Data();}
int TArtAnaLoopStatic::GetEventNumber(){return fEventNumber;}
TDirectory* TArtAnaLoopStatic::GetDirectory(){return fDirectory;}

TArtAnaLoopStatic::~TArtAnaLoopStatic()
{
  delete fAnaLoop;
  fAnaLoop = 0;
  fFileName = "";
  fDirectory = 0;
  fEventNumber = -1;
}

