
#include "TArtAnaLoopNEBULA.hh"

#include <iostream>

#include "TH1.h"
#include "TH2.h"
#include "TCanvas.h"
#include "TTree.h"
#include "TClonesArray.h"

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"

#include "TArtCalibNEBULA.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtNEBULAParameters.hh"

TArtAnaLoopNEBULA::TArtAnaLoopNEBULA()
  : TArtAnaLoop(), calibnebula(0), nebulaparameters(0)
{;}

TArtAnaLoopNEBULA::~TArtAnaLoopNEBULA()
{
  Destruct();
}

void TArtAnaLoopNEBULA::Construct()
{
  calibnebula= new TArtCalibNEBULA;
  nebulaparameters = new TArtNEBULAParameters;
  nebulaparameters->LoadParameter("db/NEBULA.xml");
}

void TArtAnaLoopNEBULA::Calculate()
{
  calibnebula->BeginEvent();
  calibnebula->ReconstructData();

  if(fAnaFile) fAnaFile->Clear();

  Int_t NumNEBULAPla = calibnebula->GetNumNEBULAPla();
  for(int i=0; i<NumNEBULAPla; ++i){
    TArtNEBULAPla* pla = calibnebula->GetNEBULAPla(i);
    if(pla->GetDataState() < 0) continue;
    if(pla->GetIHitMin() < 4) continue;

    //    TArtNEBULAPlaPara* parau = calibnebula->FindNEBULAPlaPara(pla->GetID(), "U");
    //    TArtNEBULAPlaPara* parad = calibnebula->FindNEBULAPlaPara(pla->GetID(), "D");
    //    std::cout << *parau << std::endl;
    //    std::cout << *parad << std::endl;

    int tu = pla->GetRawTU();
    int td = pla->GetRawTD();
    int qu = pla->GetRawQU();
    int qd = pla->GetRawQD();
    int id = pla->GetID();

    if(fAnaFile){
      fAnaFile->Add(kNEBULA,    kID, id, id);
      fAnaFile->Add(kNEBULA, kTURaw, id, tu);
      fAnaFile->Add(kNEBULA, kTDRaw, id, td);
      fAnaFile->Add(kNEBULA, kQURaw, id, qu);
      fAnaFile->Add(kNEBULA, kQDRaw, id, qd);
    }
  }

  if(fAnaFile) fAnaFile->Fill();

  calibnebula->EndEvent();
}

void TArtAnaLoopNEBULA::Destruct()
{
  delete calibnebula;
  calibnebula = 0;
  delete nebulaparameters;
  nebulaparameters = 0;
}

const char* TArtAnaLoopNEBULA::ClassName() const
{
  return "TArtAnaLoopNEBULA";
}

